unit ucuCadastrarLivro;


interface

uses
  utuStateMachine, utuRequest, ucuUseCase, acuFramework, nguLivro;

type
  ucCadastrarLivro = class(ucUseCase)
  private
    fLivro : ngLivro;
    fSavePoint : acPersistenceSessionSavePoint;
  public
    procedure Initialize; Override;
    procedure Finalize; Override;
  published
    procedure Effect_IncluirLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_EditarLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_BuscarLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_ExcluirLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_CancelarCadastro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_IncluirExemplar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_ExcluirExemplar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_ConcluirCadastro(piRequest: utRequest; piTransition: utTransition);
  end;

implementation

uses
  ucuManager,
  utuMessage,
  acuObject,
  utuEmprestimoOQL,
  nguExemplar,
  nguSituacao;

procedure ucCadastrarLivro.Initialize;
begin
  inherited;
end;

procedure ucCadastrarLivro.Finalize;
begin
  
  inherited;
end;

procedure ucCadastrarLivro.Effect_IncluirLivro(piRequest: utRequest; piTransition: utTransition);
var
  lFieldLivro : utField;
begin
  fSavePoint := Session.AddSavePoint;
  try
    fLivro := ngLivro.CreateNew(Self.Session);
    lFieldLivro := piRequest.Response.RootField.AddField('Livro');
    lFieldLivro.AddAttribute('OID').AsString := fLivro.IDO.AsString;
  except
    Session.RollBackToSavePoint(fSavePoint);
    raise;
  end;
  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_EditarLivro(piRequest: utRequest; piTransition: utTransition);
var
  lFieldLivroIn,
  lFieldLivroOut,
  lExemplares : utField;
  lExemplar : ngExemplar;
  lEnumExemplar : acEnumerator;
begin
  fSavePoint := Session.AddSavePoint;
  try
    lFieldLivroIn := piRequest.Message.RootField.FieldByName('Livro');
    fLivro := Session.LoadObject(ngLivro, acOIDInt.Create(lFieldLivroIn.AttributeByName('OID').AsInteger)) as ngLivro;
    lEnumExemplar := fLivro.Exemplares.GetEnumerator;
    try
      lFieldLivroOut := piRequest.Response.RootField.AddField('Livro');
      fLivro.toXML(lFieldLivroOut);
      lExemplares := lFieldLivroOut.AddField('Exemplares');
      while not lEnumExemplar.EOL do
      begin
        lExemplar := lEnumExemplar.Current as ngExemplar;
        lExemplar.toXML(lExemplares.AddField('Exemplar'));
        lEnumExemplar.MoveNext;
      end;
    finally
      lEnumExemplar.Free;
    end;
  except
    Session.RollBackToSavePoint(fSavePoint);
    raise;
  end;
  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_BuscarLivro(piRequest: utRequest; piTransition: utTransition);
var
  lFieldLivro,
  lFieldLivros : utField;
  lListLivros : acPersistentObjectList;
  lEnumLivro : acEnumerator;
  lLivro : ngLivro;
  lOQL : IObterLivros;
begin
  lFieldLivro := piRequest.Message.RootField.FieldByName('Livro');

  lListLivros := acPersistentObjectList.Create(ngLivro);
  try
    lOQL := utEmprestimoOQL.ObterLivros(Self.Session);

    if lFieldLivro.HasAttribute('Titulo')
    then lOQL.Param_titulo.Value := lFieldLivro.AttributeByName('Titulo').AsString
    else lOQL.Param_titulo.SetNull;

    if lFieldLivro.HasAttribute('ISBN')
      then lOQL.Param_isbn.Value := lFieldLivro.AttributeByName('ISBN').AsString
      else lOQL.Param_isbn.SetNull;

    if lFieldLivro.HasAttribute('Edicao')
      then lOQL.Param_edicao.Value := lFieldLivro.AttributeByName('Edicao').AsString
      else lOQL.Param_edicao.SetNull;

    lOQL.LoadList(lListLivros);

    lEnumLivro := lListLivros.GetEnumerator;
    try
      lFieldLivros := piRequest.Response.RootField.AddField('Livros');
      while not lEnumLivro.EOL do
      begin
        lLivro := lEnumLivro.Current as ngLivro;
        lLivro.toXML(lFieldLivros.AddField('Livro'));
        lEnumLivro.MoveNext;
      end;
    finally
      lEnumLivro.Free;
    end;

  finally
    lListLivros.Free;
  end;

  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_ExcluirLivro(piRequest: utRequest; piTransition: utTransition);
var
  lFieldLivro,
  lFieldLivros,
  lFieldResult : utField;
  lListLivros : acPersistentObjectList;
  lEnumLivro : acEnumerator;
  lLivro : ngLivro;
  lOQL : IContarEmprestimosLivro;
  lSavePoint : acPersistenceSessionSavePoint;
begin
  lFieldLivro := piRequest.Message.RootField.FieldByName('Livro');
  lSavePoint := Session.AddSavePoint;
  try
    lOQL := utEmprestimoOQL.ContarEmprestimosLivro(Self.Session);
    lOQL.Param_oid.Value := lFieldLivro.AttributeByName('OID').AsInteger;
    lFieldResult := utField.Create;
    try
      lOQL.LoadDataSet(lFieldResult);
      if lFieldResult.ValueByPath('DATAPACKET.ROWDATA.ROW.@C0').AsInteger > 0
        then raise EucException.Create('No  possvel excluir o livro, ele possui pelo menos um exemplar com emprstimo associado.');

      lLivro := Session.LoadObject(ngLivro, acOIDInt.Create(lFieldLivro.AttributeByName('OID').AsInteger)) as ngLivro;
      Session.DeleteObject(lLivro);
      Session.Save;
    finally
      lFieldResult.Free;
    end;
  except
    Session.RollBackToSavePoint(lSavePoint);
    raise;
  end;
  
  lListLivros := acPersistentObjectList.Create(ngLivro);
  try
    Self.Session.LoadAllInstances(ngLivro, lListLivros);
    lEnumLivro := lListLivros.GetEnumerator;
    try
      lFieldLivros := piRequest.Response.RootField.AddField('Livros');
      while not lEnumLivro.EOL do
      begin
        lLivro := lEnumLivro.Current as ngLivro;
        lLivro.toXML(lFieldLivros.AddField('Livro'));
        lEnumLivro.MoveNext;
      end;
    finally
      lEnumLivro.Free;
    end;
  finally
    lListLivros.Free;
  end;
  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_CancelarCadastro(piRequest: utRequest; piTransition: utTransition);
begin
  if Assigned(fSavePoint)
    then Session.RollBackToSavePoint(fSavePoint);
    
  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_IncluirExemplar(piRequest: utRequest; piTransition: utTransition);
var
  lFieldExemplar,
  lFieldExemplares : utField;
  lExemplar : ngExemplar;
  lEnumExemplar : acEnumerator;
begin
  lFieldExemplar := piRequest.Message.RootField.FieldByName('Exemplar');
  lExemplar := ngExemplar.CreateNew(Self.Session);
  lExemplar.Codigo.AsString := lFieldExemplar.AttributeByName('Codigo').AsString;
  lExemplar.Situacao.Partner := Session.LoadObject(ngSituacao, acOIDInt.Create(1));
  fLivro.Exemplares.Add(lExemplar);

  lFieldExemplares := piRequest.Response.RootField.AddField('Exemplares');
  lEnumExemplar := fLivro.Exemplares.GetEnumerator;
  try
    while not lEnumExemplar.EOL do
    begin
      lExemplar := lEnumExemplar.Current as ngExemplar;
      lExemplar.toXML(lFieldExemplares.AddField('Exemplar'));
      lEnumExemplar.MoveNext;
    end;
  finally
    lEnumExemplar.Free;
  end;

  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_ExcluirExemplar(piRequest: utRequest; piTransition: utTransition);
var
  lFieldExemplar,
  lFieldExemplares,
  lFieldResult : utField;
  lExemplar : ngExemplar;
  lEnumExemplar : acEnumerator;
  lOQL : IContarEmprestimosExemplar;
begin
  lFieldExemplar := piRequest.Message.RootField.FieldByName('Exemplar');
  lExemplar := Session.LoadObject(ngExemplar, acOIDInt.Create(lFieldExemplar.AttributeByName('OID').AsInteger)) as ngExemplar;
  if fLivro.Exemplares.IndexOf(lExemplar) = -1
    then raise EucException.Create('Exemplar no existe na lista de exemplares do livro.');

  lOQL := utEmprestimoOQL.ContarEmprestimosExemplar(Self.Session);
  lOQL.Param_oid.Value := lExemplar.IDO.AsVariant;
  lFieldResult := utField.Create;
  try
    lOQL.LoadDataSet(lFieldResult);
    if lFieldResult.ValueByPath('DATAPACKET.ROWDATA.ROW.@C0').AsInteger > 0
      then raise EucException.Create('No  possvel excluir o exemplar, ele possui pelo menos um emprstimo associado.');

    fLivro.Exemplares.Remove(lExemplar);
  finally
    lFieldResult.Free;
  end;
  
  lFieldExemplares := piRequest.Response.RootField.AddField('Exemplares');
  lEnumExemplar := fLivro.Exemplares.GetEnumerator;
  try
    while not lEnumExemplar.EOL do
    begin
      lExemplar := lEnumExemplar.Current as ngExemplar;
      lExemplar.toXML(lFieldExemplares.AddField('Exemplar'));
      lEnumExemplar.MoveNext;
    end;
  finally
    lEnumExemplar.Free;
  end;

  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_ConcluirCadastro(piRequest: utRequest; piTransition: utTransition);
var
  lSavePoint : acPersistenceSessionSavePoint;
  lFieldLivroIn,
  lFieldLivroOut,
  lFieldExemplares : utField;
  lExemplar : ngExemplar;
  lEnumExemplar : acEnumerator; 
begin
  lSavePoint := Self.Session.AddSavePoint;
  try
    lFieldLivroIn := piRequest.Message.RootField.FieldByName('Livro');
    fLivro.Titulo.AsString := lFieldLivroIn.AttributeByName('Titulo').AsString;
    fLivro.ISBN.AsString := lFieldLivroIn.AttributeByName('ISBN').AsString;
    fLivro.Edicao.AsString := lFieldLivroIn.AttributeByName('Edicao').AsString;
    
    Session.Save;
    
    lFieldLivroOut := piRequest.Response.RootField.AddField('Livro');
    fLivro.toXML(lFieldLivroOut);
    lFieldExemplares := lFieldLivroOut.AddField('Exemplares');
    lEnumExemplar := fLivro.Exemplares.GetEnumerator;
    try
      while not lEnumExemplar.EOL do
      begin
        lExemplar := lEnumExemplar.Current as ngExemplar;
        lExemplar.toXML(lFieldExemplares.AddField('Exemplar'));
        lEnumExemplar.MoveNext;
      end;
    finally
      lEnumExemplar.Free;
    end;
  except
    Session.RollBackToSavePoint(lSavePoint);
    raise;
  end; 
  piRequest.Processed := True;
end;


initialization

  gucManager.RegisterClass(3226,ucCadastrarLivro);

end.
