unit ucuConsultarEmprestimo;


interface

uses
  utuStateMachine, utuRequest, ucuUseCase, acuFramework;

type
  ucConsultarEmprestimo = class(ucUseCase)
  public
    procedure Initialize; Override;
    procedure Finalize; Override;
  published
    procedure Effect_Consultar(piRequest: utRequest; piTransition: utTransition);
  end;

implementation

uses
  ucuManager,
  utuMessage,
  acuObject,
  utuEmprestimoOQL,
  nguUsuario,
  nguLivro,
  nguExemplar,
  nguEmprestimo,
  SysUtils,
  DateUtils;

procedure ucConsultarEmprestimo.Initialize;
begin
  inherited;
end;

procedure ucConsultarEmprestimo.Finalize;
begin
  
  inherited;
end;

procedure ucConsultarEmprestimo.Effect_Consultar(piRequest: utRequest; piTransition: utTransition);
var
  lEnumEmprestimo : acEnumerator;
  lExemplar : ngExemplar;
  lEmprestimo : ngEmprestimo;
  lLivro : ngLivro;
  lListEmprestimos : acPersistentObjectList;
  lUsuario : ngUsuario;
  lOQL: IObterEmprestimos;
  lFieldEmprestimos,
  lFieldEmprestimo,
  lFieldUsuario,
  lFieldLivro,
  lFieldExemplar,
  lFieldUsuarioIn : utField;
begin
  lOQL := utEmprestimoOQL.ObterEmprestimos(Self.Session);
  lListEmprestimos := acPersistentObjectList.Create(ngEmprestimo);
  try
    lFieldUsuarioIn := piRequest.Message.RootField.FieldByName('Usuario');
    
    if lFieldUsuarioIn.HasAttribute('OID')
      then lOQL.Param_oid.Value := lFieldUsuarioIn.AttributeByName('OID').AsInteger
      else lOQL.Param_oid.SetNull;

    if lFieldUsuarioIn.HasAttribute('Nome')
      then lOQL.Param_nome.Value := lFieldUsuarioIn.AttributeByName('Nome').AsString
      else lOQL.Param_nome.SetNull;

    if lFieldUsuarioIn.HasAttribute('CPF')
      then lOQL.Param_cpf.Value := lFieldUsuarioIn.AttributeByName('CPF').AsString
      else lOQL.Param_cpf.SetNull;

    lOQL.LoadList(lListEmprestimos);
    
    lEnumEmprestimo := lListEmprestimos.GetEnumerator;
    try
      lFieldEmprestimos := piRequest.Response.RootField.AddField('Emprestimos');
      while not lEnumEmprestimo.EOL do
      begin
        //Tag de Emprestimo
        lEmprestimo := lEnumEmprestimo.Current as ngEmprestimo;
        lFieldEmprestimo := lFieldEmprestimos.AddField('Emprestimo');
        lEmprestimo.toXML(lFieldEmprestimo);
        //Tag de Usurio
        lUsuario := lEmprestimo.Usuario.Partner as ngUsuario;
        lFieldUsuario := lFieldEmprestimo.AddField('Usuario');
        lUsuario.toXML(lFieldUsuario);
        //Tag de Exemplar
        lExemplar := lEmprestimo.Exemplar.Partner as ngExemplar;
        lFieldExemplar := lFieldEmprestimo.AddField('Exemplar');
        lExemplar.toXML(lFieldExemplar);
        //Tag de Livro
        lFieldLivro := lFieldExemplar.AddField('Livro');
        lLivro := lExemplar.Livro.Partner as ngLivro;
        lLivro.toXML(lFieldLivro);
        lEnumEmprestimo.MoveNext;
      end;
    finally
      lEnumEmprestimo.Free;
    end;
  finally
    lListEmprestimos.Free;
  end;

  
  piRequest.Processed := True;
end;


initialization

  gucManager.RegisterClass(3222,ucConsultarEmprestimo);

end.
