unit ucuEfetuarEmprestimo;


interface

uses
  utuStateMachine, utuRequest, ucuUseCase, nguUsuario, acuFramework;

type
  ucEfetuarEmprestimo = class(ucUseCase)
  public
    procedure Initialize; Override;
    procedure Finalize; Override;
  private
    fSavePoint : acPersistenceSessionSavePoint;
    fUsuario : ngUsuario;
    fListExemplares : acPersistentObjectList;
  published
    procedure Effect_PesquisarUsuario(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_EscolherUsuario(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_RealizarEmprestimo(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_PesquisarExemplar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_RemoverExemplar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_Confirmar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_CancelarConfirmacao(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_CancelarEscolhaExemplar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_SelecionarExemplar(piRequest: utRequest; piTransition: utTransition);
  end;

implementation

uses
  ucuManager,
  utuMessage {define utField},
  acuObject {define acEnumerator},
  utuEmprestimoOQL,
  nguLivro,
  nguExemplar,
  nguEmprestimo,
  SysUtils,
  DateUtils;

procedure ucEfetuarEmprestimo.Initialize;
begin
  inherited;
  fListExemplares := acPersistentObjectList.Create(ngExemplar);
end;

procedure ucEfetuarEmprestimo.Finalize;
begin
  fListExemplares.Free;
  inherited;
end;

procedure ucEfetuarEmprestimo.Effect_PesquisarUsuario(piRequest: utRequest; piTransition: utTransition);
var
  lFieldUsuario,
  lFieldUsuarios: utField;
  lOQL: IObterUsuarios;
  lListUsuarios: acPersistentObjectList;
  lEnumUsuario: acEnumerator;
  lUsuario: ngUsuario;
begin
  lFieldUsuario := piRequest.Message.RootField.FieldByName('Usuario');

  lOQL := utEmprestimoOQL.ObterUsuarios(Self.Session);

  lFieldUsuarios := piRequest.Response.RootField.AddField('Usuarios');

  lListUsuarios := acPersistentObjectList.Create(ngUsuario);
  try

    if lFieldUsuario.HasAttribute('Nome')
      then lOQL.Param_nome.Value := lFieldUsuario.AttributeByName('Nome').AsString
      else lOQL.Param_nome.SetNull;

    if lFieldUsuario.HasAttribute('CPF')
      then lOQL.Param_cpf.Value := lFieldUsuario.AttributeByName('CPF').AsString
      else lOQL.Param_cpf.SetNull;

    lOQL.LoadList(lListUsuarios);

    lEnumUsuario := acEnumerator.Create(lListUsuarios);
    try
      while not lEnumUsuario.EOL do
      begin
        lUsuario := lEnumUsuario.Current as ngUsuario;
        lUsuario.toXML(lFieldUsuarios.AddField('Usuario'));
        lEnumUsuario.MoveNext;
      end;
    finally
      lEnumUsuario.Free;
    end;


  finally
    lListUsuarios.Free;
  end;

  piRequest.Processed := True;
end;

procedure ucEfetuarEmprestimo.Effect_EscolherUsuario(piRequest: utRequest; piTransition: utTransition);
var
  lFieldUsuario: utField;
begin
  lFieldUsuario := piRequest.Message.RootField.FieldByName('Usuario');
  try
    fUsuario := Self.Session.LoadObject(ngUsuario, acOIDInt.Create(lFieldUsuario.AttributeByName('OID').AsInteger)) as ngUsuario;
  except
    on e : acObjectNotFoundException do
      raise EucException.Create('Usurio no encontrado.');
    else
      raise EucException.Create('Sistema no pode determinar se o usurio pode ser encontrado.');
  end;
  fUsuario.toXML(piRequest.Response.RootField.AddField('Usuario'));
  piRequest.Processed := True;
end;

procedure ucEfetuarEmprestimo.Effect_RealizarEmprestimo(piRequest: utRequest; piTransition: utTransition);
var
  lEnumExemplar : acEnumerator;
  lExemplar : ngExemplar;
  lEmprestimo : ngEmprestimo;
  lLivro : ngLivro;
  lFieldEmprestimos,
  lFieldEmprestimo,
  lFieldUsuario,
  lFieldLivro,
  lFieldExemplar : utField;
begin
  fSavePoint := Self.Session.AddSavePoint;
  try
    lEnumExemplar := fListExemplares.GetEnumerator;
    try
      lFieldUsuario := piRequest.Response.RootField.AddField('Usuario');
      fUsuario.toXML(lFieldUsuario);
      lFieldEmprestimos := piRequest.Response.RootField.AddField('Emprestimos');
      while not lEnumExemplar.EOL do
      begin
        lExemplar := lEnumExemplar.Current as ngExemplar;
        lEmprestimo := ngEmprestimo.CreateNew(Self.Session);
        lEmprestimo.Usuario.Partner := fUsuario;
        lEmprestimo.Exemplar.Partner := lExemplar;
        lEmprestimo.DataInicio.Value := Now;
        lEmprestimo.DataDevolucaoPrevista.Value := IncWeek(lEmprestimo.DataInicio.Value, 3);
        //Tag de Emprestimo
        lFieldEmprestimo := lFieldEmprestimos.AddField('Emprestimo');
        lEmprestimo.toXML(lFieldEmprestimo);
        //Tag de Exemplar
        lFieldExemplar := lFieldEmprestimo.AddField('Exemplar');
        lExemplar.toXML(lFieldExemplar);
        //Tag de Livro
        lFieldLivro := lFieldExemplar.AddField('Livro');
        lLivro := lExemplar.Livro.Partner as ngLivro;
        lLivro.toXML(lFieldLivro);
        lEnumExemplar.MoveNext;
      end;
    finally
      lEnumExemplar.Free;
    end;

  except
    Self.Session.RollBackToSavePoint(fSavePoint);
    raise;
  end;
  piRequest.Processed := True;
end;

procedure ucEfetuarEmprestimo.Effect_PesquisarExemplar(piRequest: utRequest; piTransition: utTransition);
var
  lFieldExemplar,
  lFieldLivros,
  lFieldCurrentLivro,
  lFieldCurrentExemplares,
  lFieldLivro : utField;
  lListExemplares : acPersistentObjectList;
  lEnumExemplar : acEnumerator;
  lExemplar : ngExemplar;
  lLastLivro: ngLivro;
  lOQL : IObterExemplares;
begin
  lLastLivro := nil;
  lFieldCurrentExemplares := nil;
  lFieldLivro := nil;

  lFieldExemplar := piRequest.Message.RootField.FieldByName('Exemplar');
  if lFieldExemplar.HasField('Livro')
  then
    lFieldLivro := lFieldExemplar.FieldByName('Livro');

  lOQL := utEmprestimoOQL.ObterExemplares(Self.Session);

  lFieldLivros := piRequest.Response.RootField.AddField('Livros');

  lListExemplares := acPersistentObjectList.Create(ngExemplar);
  try

    if lFieldExemplar.HasAttribute('Codigo')
      then lOQL.Param_codigo.Value := lFieldExemplar.AttributeByName('Codigo').AsString
      else lOQL.Param_codigo.SetNull;

    if lFieldExemplar.HasField('Livro')
      then
      begin
        if lFieldLivro.HasAttribute('ISBN')
          then lOQL.Param_isbn.Value := lFieldLivro.AttributeByName('ISBN').AsString
          else lOQL.Param_isbn.SetNull;

        if lFieldLivro.HasAttribute('Titulo')
          then lOQL.Param_titulo.Value := lFieldLivro.AttributeByName('Titulo').AsString
          else lOQL.Param_titulo.SetNull;
      end
      else
      begin
        lOQL.Param_isbn.SetNull;
        lOQL.Param_titulo.SetNull;
      end;
    lOQL.LoadList(lListExemplares);

    lEnumExemplar := acEnumerator.Create(lListExemplares);
    try
      while not lEnumExemplar.EOL do
      begin
        lExemplar := lEnumExemplar.Current as ngExemplar;
        if not Assigned(lLastLivro)
          then
          begin
              lLastLivro := lExemplar.Livro.Partner as ngLivro;
              lFieldCurrentLivro := lFieldLivros.AddField('Livro');
              lLastLivro.toXML(lFieldCurrentLivro);
              lFieldCurrentExemplares := lFieldCurrentLivro.AddField('Exemplares');
          end
          else
            if (not lExemplar.Livro.Partner.Equals(lLastLivro))
              then
              begin
                lLastLivro := lExemplar.Livro.Partner as ngLivro;
                lFieldCurrentLivro := lFieldLivros.AddField('Livro');
                lLastLivro.toXML(lFieldCurrentLivro);
                lFieldCurrentExemplares := lFieldCurrentLivro.AddField('Exemplares');
              end;
        lExemplar.toXML(lFieldCurrentExemplares.AddField('Exemplar'));

        lEnumExemplar.MoveNext;
      end;
    finally
      lEnumExemplar.Free;
    end;


  finally
    lListExemplares.Free;
  end;
  
  piRequest.Processed := True;
end;

procedure ucEfetuarEmprestimo.Effect_RemoverExemplar(piRequest: utRequest; piTransition: utTransition);
var
  lFieldExemplar,
  lFieldExemplaresOut,
  lFieldExemplarOut : utField;
  lExemplar : ngExemplar;
  lLivro : ngLivro;
  lEnumExemplar : acEnumerator;
begin
  lFieldExemplar := piRequest.Message.RootField.FieldByName('Exemplar');
  try
    lExemplar := Self.Session.LoadObject(ngExemplar, acOIDInt.Create(lFieldExemplar.AttributeByName('OID').AsInteger)) as ngExemplar;
  except
    on e : acObjectNotFoundException do
      raise EucException.Create('Exemplar no encontrado.');
    else
      raise EucException.Create('Sistema no pode determinar se o exemplar pode ser encontrado.');
  end;

  if (fListExemplares.IndexOf(lExemplar) <> -1)
    then
      fListExemplares.Extract(lExemplar)
    else
      raise EucException.Create('Exemplar no existe na lista de emprstimos.');

  lFieldExemplaresOut := piRequest.Response.RootField.AddField('Exemplares');
  lEnumExemplar := acEnumerator.Create(fListExemplares);
  try
    while not lEnumExemplar.EOL do
    begin
      lExemplar := lEnumExemplar.Current as ngExemplar;
      lFieldExemplarOut := lFieldExemplaresOut.AddField('Exemplar');
      lExemplar.toXML(lFieldExemplarOut);
      lLivro := lExemplar.Livro.Partner as ngLivro;
      lLivro.toXML(lFieldExemplarOut.AddField('Livro'));
      lEnumExemplar.MoveNext;
    end;
  finally
    lEnumExemplar.Free;
  end;  
  
  piRequest.Processed := True;
end;

procedure ucEfetuarEmprestimo.Effect_Confirmar(piRequest: utRequest; piTransition: utTransition);
var
  lSavePoint : acPersistenceSessionSavePoint; 
begin
  lSavePoint := Self.Session.AddSavePoint;
  try
    Session.Save;
  except
    Session.RollBackToSavePoint(lSavePoint);
    raise;
  end;
  piRequest.Processed := True;
end;

procedure ucEfetuarEmprestimo.Effect_CancelarConfirmacao(piRequest: utRequest; piTransition: utTransition);
var
  lFieldExemplaresOut,
  lFieldExemplarOut : utField;
  lExemplar : ngExemplar;
  lLivro : ngLivro;
  lEnumExemplar : acEnumerator;
begin
  if Assigned(fSavePoint)
    then
      Self.Session.RollBackToSavePoint(fSavePoint);
  lFieldExemplaresOut := piRequest.Response.RootField.AddField('Exemplares');
  lEnumExemplar := acEnumerator.Create(fListExemplares);
  try
    while not lEnumExemplar.EOL do
    begin
      lExemplar := lEnumExemplar.Current as ngExemplar;
      lFieldExemplarOut := lFieldExemplaresOut.AddField('Exemplar');
      lExemplar.toXML(lFieldExemplarOut);
      lLivro := lExemplar.Livro.Partner as ngLivro;
      lLivro.toXML(lFieldExemplarOut.AddField('Livro'));
      lEnumExemplar.MoveNext;
    end;
  finally
    lEnumExemplar.Free;
  end;
  
  piRequest.Processed := True;
end;

procedure ucEfetuarEmprestimo.Effect_CancelarEscolhaExemplar(piRequest: utRequest; piTransition: utTransition);
var
  lFieldExemplaresOut,
  lFieldExemplarOut : utField;
  lExemplar : ngExemplar;
  lLivro : ngLivro;
  lEnumExemplar : acEnumerator;
begin
  lFieldExemplaresOut := piRequest.Response.RootField.AddField('Exemplares');
  lEnumExemplar := acEnumerator.Create(fListExemplares);
  try
    while not lEnumExemplar.EOL do
    begin
      lExemplar := lEnumExemplar.Current as ngExemplar;
      lFieldExemplarOut := lFieldExemplaresOut.AddField('Exemplar');
      lExemplar.toXML(lFieldExemplarOut);
      lLivro := lExemplar.Livro.Partner as ngLivro;
      lLivro.toXML(lFieldExemplarOut.AddField('Livro'));
      lEnumExemplar.MoveNext;
    end;
  finally
    lEnumExemplar.Free;
  end; 
  
  piRequest.Processed := True;
end;

procedure ucEfetuarEmprestimo.Effect_SelecionarExemplar(piRequest: utRequest; piTransition: utTransition);
var
  lFieldExemplar,
  lFieldExemplaresOut,
  lFieldExemplarOut : utField;
  lExemplar : ngExemplar;
  lLivro : ngLivro;
  lEnumExemplar : acEnumerator;
begin
  lFieldExemplar := piRequest.Message.RootField.FieldByName('Exemplar');
  try
    lExemplar := Self.Session.LoadObject(ngExemplar, acOIDInt.Create(lFieldExemplar.AttributeByName('OID').AsInteger)) as ngExemplar;
  except
    on e : acObjectNotFoundException do
      raise EucException.Create('Exemplar no encontrado.');
    else
      raise EucException.Create('Sistema no pode determinar se o exemplar pode ser encontrado.');
  end;

  if fListExemplares.IndexOf(lExemplar) = -1
  then
    fListExemplares.Add(lExemplar)
  else
    raise EucException.Create('Exemplar j est na lista para emprstimo.');
  lFieldExemplaresOut := piRequest.Response.RootField.AddField('Exemplares');
  lEnumExemplar := acEnumerator.Create(fListExemplares);
  try
    while not lEnumExemplar.EOL do
    begin
      lExemplar := lEnumExemplar.Current as ngExemplar;
      lFieldExemplarOut := lFieldExemplaresOut.AddField('Exemplar');
      lExemplar.toXML(lFieldExemplarOut);
      lLivro := lExemplar.Livro.Partner as ngLivro;
      lLivro.toXML(lFieldExemplarOut.AddField('Livro'));
      lEnumExemplar.MoveNext;
    end;
  finally
    lEnumExemplar.Free;
  end;
  piRequest.Processed := True;
end;


initialization

  gucManager.RegisterClass(3220,ucEfetuarEmprestimo);

end.
